/**@file CamView/MainFrame.cpp: Form with main frame for Camera viewer. */

#include <wx/wxprec.h>
#include <wx/confbase.h>
#ifndef WX_PRECOMP
 #include <wx/wx.h>
#endif
#include <wx/dcbuffer.h> // for preview 

#include "MainFrame.h"
#include "../library/AboutDialog.h"
#include "VideoSetting.h"

#include "RT_EXEC.h"
#include "../library/wxTools.h"
#include "../library/PntwxTools.h"
#include "../library/hardware/OsSupp.h"

#define Image RsImage
#include "../library/images/Raster.h"
#undef Image


#define AUTO_FORM_REPAINT 32	//[ms]


#define DEFINE_ONLY
#include "CamView.rc"
#undef DEFINE_ONLY


/** This string should be used inside all message boxes. */
const TCHAR *APP_NAME = _T("Camera demo app");
const TCHAR *AboutText =
     _T("Camera demo app\n(c) 2009 Jaroslav Fojtik\nVersion " VERSIONSTR);


/** Create main frame.
 *  @param[in] title The caption to be displayed on the frame's title bar
 *  @param[in] pos The window position
 *  @param[in] size The window size      */
BnPMainFrame::BnPMainFrame(const wxString& title, const wxPoint& pos, const wxSize& size):     
     wxFrame((wxFrame*)NULL, -1, title, pos, size, wxTAB_TRAVERSAL | wxDEFAULT_FRAME_STYLE),     
     mPreviewTimer(this, ID_TIMER), m_bmpBkg(NULL)   
{ 
  m_container.SetContainerWindow(this);

    // Make a menubar
  menuBar = new wxMenuBar;

    // create Hardware
  menuFile = new wxMenu;
  menuBar->Append(menuFile, _T("&File"));    

  menuFile->Append(ID_SAVEAS, _T("&Save img as .."), _T("Save image 1 into a file specified."));
  menuFile->Append(ID_LOAD, _T("&Load image"), _T("Stop camera1 and load an image given."));
  menuFile->Append(ID_SAVEAS2,_T("&Save img2 as .."), _T("Save image 2 into a file specified."));

  menuFile->Append(ID_SAVECONFIG, _T("&Save Config"), _T("Save configuration to ini file."));
  menuFile->Append(ID_RELOADCONFIG, _T("&Reload Config"), _T("Reload configuration from ini file."));   


  menuSettings = new wxMenu;
  menuBar->Append(menuSettings, _T("&Settings"));    

  menuSettings->Append(ID_VIDEO, _T("&Camera 1"), _T("Setup of 1st camera."));
  menuSettings->Append(ID_VIDEO2, _T("&Camera 2"), _T("Setup of 2nd camera."));
  menuSettings->AppendSeparator(); 
  menuSettings->Append(ID_SET_DEFAULT, _T("&Set Factory Defaults"), _T("Set default values for all options."));

    // create Camera
  menuDisplay = new wxMenu;
  menuBar->Append(menuDisplay, _T("&Camera"));
		// append menu entries    
  CamBW =   menuDisplay->AppendRadioItem(ID_PAINT_BW, _T("&Thresholded"), _T("Paint B&W image."));
  CamGray = menuDisplay->AppendRadioItem(ID_PAINT_GRAY, _T("&Raw gray"), _T("Paint gray image."));
  CamColor= menuDisplay->AppendRadioItem(ID_PAINT_COLOR, _T("&Raw color"), _T("Paint color image."));
  menuDisplay->AppendSeparator(); 
  CamPaint = menuDisplay->AppendCheckItem(ID_PAINT_STOP, _T("&Stop image"), _T("Stop image."));
  CamFilter= menuDisplay->AppendCheckItem(ID_FILTER_IMG, _T("&Filter image"), _T("Filter image data."));
  CamRectangle=menuDisplay->AppendCheckItem(ID_RECTANGLE, _T("&Rectangle"), _T("Cut image to rectangle shape."));
  menuDisplay->Append(ID_HISTOGRAM, _T("&Histogram"), _T("Show histogram."));

    // create Camera
  menuDisplay2 = new wxMenu;
  menuBar->Append(menuDisplay2, _T("&Camera 2"));
		// append menu entries    
  CamBW2 =   menuDisplay2->AppendRadioItem(ID_PAINT_BW2, _T("&Thresholded"), _T("Paint B&W image."));
  CamGray2 = menuDisplay2->AppendRadioItem(ID_PAINT_GRAY2, _T("&Raw gray"), _T("Paint gray image."));
  CamColor2= menuDisplay2->AppendRadioItem(ID_PAINT_COLOR2, _T("&Raw color"), _T("Paint color image."));
  menuDisplay2->AppendSeparator(); 
  CamPaint2 = menuDisplay2->AppendCheckItem(ID_PAINT_STOP2, _T("&Stop image"), _T("Stop image."));
  CamFilter2= menuDisplay2->AppendCheckItem(ID_FILTER_IMG2, _T("&Filter image"), _T("Filter image data."));
  CamRectangle2=menuDisplay2->AppendCheckItem(ID_RECTANGLE2, _T("&Rectangle"), _T("Cut image to rectangle shape."));
  menuDisplay2->Append(ID_HIDE2, _T("&Hide"), _T("Hide camera canvas."));


    // create Help
  menuHelp = new wxMenu;
  menuBar->Append(menuHelp, _T("&Help"));
    // append menu entries
  menuHelp->Append(ID_ABOUT, _T("&About"), _T("About the camera viewer."));
   
    // Associate the menu bar with the frame
  SetMenuBar(menuBar);

    // create frame statusbar
  CreateStatusBar();
    // set statusbar text
  SetStatusText("Welcome to camera viewer!");
    

    ////************* Arrange widgets **************************************
  BnPMainFrame* itemFrame = this;    

      /** create vertical stack with  ball plate and combo box. */
  SzrChart = new wxBoxSizer(wxHORIZONTAL);

  camCanvas = new CameraPreview(itemFrame,&MRT.CT);	//chart with camera preview
  camCanvas2 = new CameraPreview(itemFrame,&MRT.CT2);	//chart with camera preview
        
  szitemChart = SzrChart->Add(camCanvas, 1, wxGROW|wxALL|wxSHAPED, 5);
  szitemChart2 = SzrChart->Add(camCanvas2, 1, wxGROW|wxALL|wxSHAPED, 5);

  Szr_Top = new wxBoxSizer(wxHORIZONTAL); //Main sizer for Main frame
  itemFrame->SetSizer(Szr_Top);    

  Szr_Top->Add(SzrChart,1, wxGROW|wxALL, 0);
	
  #ifdef __WXMSW__
  SetIcon(wxICON(ball));
  #endif		/// \todo set icon also for other OS
    
    // restore the control's values from the config    
  wxConfigBase *pConfig = wxConfigBase::Get();    
  pConfig->SetPath("/MainFrame");

    // restore frame position and size
  int x = pConfig->Read("x", 50),
      y = pConfig->Read("y", 50),
      w = pConfig->Read("w", 510),
      h = pConfig->Read("h", 430);
  pConfig->SetPath(_T("../"));		// SetPath() understands ".."
    
  Move(x, y);    
  const wxSize & minSize = wxSize(300, 180);
  itemFrame->SetSizeHints(minSize);
  const wxSize & appSize = wxSize(w, h);
  itemFrame->SetSize(appSize);
  
  OnLoadConfig();  

  if(MRT.CT.Camera==MRT.CT2.Camera || MRT.CT2.Camera==NULL)
    camCanvas2->Hide();

  m_bmpBkg=new wxBitmap;
  WxLoadBitmap(m_bmpBkg,_T("bcgmodra.bmp"));
  if(!m_bmpBkg->Ok())
      {delete m_bmpBkg; m_bmpBkg=NULL;}
  

  SetAutoLayout(true);
  Layout();

	/* Solve amount of items inside combo box. */
  wxCommandEvent EvtChRes(wxEVT_CAMERARESCHANGED,GetId());
  EvtChRes.SetEventObject(this);
  OnResolutionChange(EvtChRes);
    //GetEventHandler()->ProcessEvent(EvtChRes);

	//video refresh rate 200ms - THIS VALUE DIFFERS FROM REAL TIME PERIOD
  mPreviewTimer.Start(AUTO_FORM_REPAINT);
}


BnPMainFrame::~BnPMainFrame() 
{
  if(m_bmpBkg) {delete m_bmpBkg;m_bmpBkg=NULL;}  //release wallpaper

    // save the frame position
  wxConfigBase *pConfig = wxConfigBase::Get();
  if(pConfig==NULL) return;      

  if(!IsIconized())	//avoid storing garbage when iconised
    {
    int x, y, w, h;
    GetSize(&w, &h);
    GetPosition(&x, &y);
    pConfig->Write("/MainFrame/x", (long)x);
    pConfig->Write("/MainFrame/y", (long)y);
    pConfig->Write("/MainFrame/w", (long)w);
    pConfig->Write("/MainFrame/h", (long)h);
    }    
}


/**
 * // ----------------------------------------------------------------------------
 * // event tables and other macros for wxWidgets
 * // ----------------------------------------------------------------------------
 *
 * // the event tables connect the wxWidgets events with the functions (event
 * // handlers) which process them. It can be also done at run-time, but for the
 * // simple menu events like this the static method is much simpler.
 */
BEGIN_EVENT_TABLE(BnPMainFrame, wxFrame)
    EVT_TIMER(ID_TIMER,         BnPMainFrame::OnTimer)  //place timer first to speedup table search.
    EVT_PAINT(                  BnPMainFrame::OnPaint)
    EVT_ERASE_BACKGROUND(       BnPMainFrame::OnEraseBackground)

    EVT_MENU(ID_SAVEAS,         BnPMainFrame::OnSaveAs)
    EVT_MENU(ID_LOAD,           BnPMainFrame::OnLoad)
    EVT_MENU(ID_SAVEAS2,	BnPMainFrame::OnSaveAs2)
    EVT_MENU(ID_ABOUT,          BnPMainFrame::OnAbout)
    EVT_MENU(ID_PAINT_BW,	BnPMainFrame::OnBlackWhite)
    EVT_MENU(ID_HIDE2,          BnPMainFrame::OnHide2)
    EVT_MENU(ID_PAINT_GRAY,     BnPMainFrame::OnGray)
    EVT_MENU(ID_PAINT_COLOR,    BnPMainFrame::OnColor)
    EVT_MENU(ID_PAINT_STOP,     BnPMainFrame::OnPaintStop)
    EVT_MENU(ID_FILTER_IMG,     BnPMainFrame::OnPaintFilter)
    EVT_MENU(ID_RECTANGLE,      BnPMainFrame::OnRectangle)
    EVT_MENU(ID_HISTOGRAM,      BnPMainFrame::OnHistogram)
    EVT_MENU(ID_PAINT_BW2,	BnPMainFrame::OnBlackWhite2)
    EVT_MENU(ID_PAINT_GRAY2,	BnPMainFrame::OnGray2)
    EVT_MENU(ID_PAINT_COLOR2,	BnPMainFrame::OnColor2)
    EVT_MENU(ID_PAINT_STOP2,	BnPMainFrame::OnPaintStop2)
    EVT_MENU(ID_FILTER_IMG2,	BnPMainFrame::OnPaintFilter2)
    EVT_MENU(ID_RECTANGLE2,	BnPMainFrame::OnRectangle2)
    EVT_MENU(ID_VIDEO,          BnPMainFrame::OnVideo)
    EVT_MENU(ID_VIDEO2,         BnPMainFrame::OnVideo2)
    EVT_MENU(ID_SAVECONFIG,     BnPMainFrame::OnSaveConfig)
    EVT_MENU(ID_RELOADCONFIG,   BnPMainFrame::OnReloadConfig)
    //EVT_MOTION(               BnPMainFrame::OnMouseMove)        
    //EVT_BUTTON(ID_BUTTON_RESET, BnPMainFrame::OnCalibrate)
    //// There is conflict between RESET and SETDEFAULT Original program is confusing
    EVT_MENU(ID_SET_DEFAULT,    BnPMainFrame::OnSetDefault)
    EVT_SIZE(                   BnPMainFrame::OnSize)

//    EVT_COMMAND(-1, wxEVT_FORMCLOSED, BnPMainFrame::OnPlotClose) ///< This signals that plotter form is closed
    EVT_COMMAND(-1, wxEVT_CAMERARESCHANGED, BnPMainFrame::OnResolutionChange) ///< Camera resolution has been switched
    EVT_COMMAND(-1, wxEVT_CAMERACHANGED, BnPMainFrame::OnCameraChange)        ///< Camera has been changed

    WX_EVENT_TABLE_CONTROL_CONTAINER(BnPMainFrame) 
END_EVENT_TABLE()


WX_DELEGATE_TO_CONTROL_CONTAINER(BnPMainFrame)


/** Plot window was shut down - Reactive the plot button */
void BnPMainFrame::OnPlotClose(wxCommandEvent & WXUNUSED(event))
{
//  BtnPlot->Enable(true);
}


/** Repaint wallpaper of BP GUI. */
void BnPMainFrame::OnPaint(wxPaintEvent &WXUNUSED(event))
{
int width;
int height;

  wxPaintDC dc(this);
  GetClientSize(&width, &height);

  if(m_bmpBkg && !ProblemsWithBrush)
    {        
    wxBrush BackBrush(*m_bmpBkg);
    BackBrush.SetStyle(wxSTIPPLE);
    
    dc.SetBrush(BackBrush);
    dc.SetLogicalFunction(wxCOPY);	//paint rectangle without a frame
    dc.SetPen(*wxTRANSPARENT_PEN); 	// - also avoid crash on W9x systems
    dc.DrawRectangle(0,0,width,height);
    }
  else dc.Clear();
}


/** Periodic timer actualises a form and repaints a bitmap from camera. */
void BnPMainFrame::OnTimer(wxTimerEvent & event)
{
bool Cam1Grab=false;

  if(CamPaint==NULL || CamPaint->IsChecked()) goto NoCamera1;

  if(camCanvas->CT->Camera == NULL) goto NoCamera1;
  if(camCanvas->CT->Camera->GetLastScan()==LastImageTimestamp)
    {
    if(camCanvas->CT==camCanvas2->CT) return; // cameras are same
    goto NoCamera1;
    }
  LastImageTimestamp = camCanvas->CT->Camera->GetLastScan();

  try {
      if(!MRT.TimerRunning()) 
	 {  //capture image only if it is not captured from RT
	 camCanvas->CT->Execute();
	 }

      if(camCanvas)
        if(camCanvas->IsShown())
          camCanvas->CaptureImage();
      }
  catch(...) {}  

  if(camCanvas) 
    if(camCanvas->IsShown())
      {
      camCanvas->OnPreviewImageTimer(event);  //Update BP frame
      Cam1Grab = true;
      }

NoCamera1:
  if(CamPaint2==NULL || CamPaint2->IsChecked()) return;
  if(camCanvas2->CT->Camera == NULL) return;  
  if(camCanvas->CT!=camCanvas2->CT || !Cam1Grab)
    if(camCanvas2->CT->Camera->GetLastScan()==LastImageTimestamp2) return;
  LastImageTimestamp2 = camCanvas2->CT->Camera->GetLastScan();

  try {
      if(!MRT.TimerRunning()) 
	 {  //capture image only if it is not captured from RT
         if(camCanvas->CT!=camCanvas2->CT  || !Cam1Grab)
	   if(camCanvas2->IsShown())
	     camCanvas2->CT->Execute();
	 }

      if(camCanvas2)
        if(camCanvas2->IsShown())
          camCanvas2->CaptureImage();
      }
  catch(...) {}  

  if(camCanvas2)
    if(camCanvas2->IsShown())
      camCanvas2->OnPreviewImageTimer(event);  //Update BP frame

}


void BnPMainFrame::OnHide2(wxCommandEvent& WXUNUSED(event))
{ 
  if(camCanvas2)
    if(camCanvas2->IsShown())
      {
      camCanvas2->Hide();
      SetAutoLayout(true);
      Layout();
      Refresh();
      }
}  


/** Opens an about dialog 
 *  @param[in] event Unused event  */
void BnPMainFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{ 
  AboutDialog *dlg = new AboutDialog(this,"ballshow.bmp","About Camera Viewer");
  dlg->SetText(AboutText);
  dlg->wxDialog::ShowModal();
}


/** Store everything into a configuration file. */
void BnPMainFrame::OnSaveConfig(wxCommandEvent & WXUNUSED(event))
{
wxString Str;

  wxConfigBase *pConfig = wxConfigBase::Get();
  if ( pConfig == NULL ) return;  

  // save a sampling period
  pConfig->SetPath("/HwSetting/");
  pConfig->Write(_T("timer"), MRT.GetPeriodMs());   
  pConfig->SetPath("../");

  // save camera properties
  pConfig->SetPath("/camera1/");
  if(MRT.threshold!=NULL)
    pConfig->Write(_T("threshold"), MRT.threshold->GetDouble());

  if(MRT.CT.Camera)
    if(MRT.CT.Camera->GetWidth()>0 && MRT.CT.Camera->GetHeight()>0)
      {
      pConfig->Write(_T("width"), MRT.CT.Camera->GetWidth());
      pConfig->Write(_T("height"), MRT.CT.Camera->GetHeight());
      pConfig->Write(_T("depth"), MRT.CT.Camera->GetDepth());
      if(MRT.CT.Camera->GetFrameRate()>0)
        pConfig->Write(_T("framerate"), MRT.CT.Camera->GetFrameRate());
      }
   if(camCanvas)
      pConfig->Write(_T("viewmode"), camCanvas->GetPreviewStatus());
   pConfig->SetPath("../"); 


// save camera2 properties
  pConfig->SetPath("/camera2/");
//  if(MRT.threshold2!=NULL)
//    pConfig->Write(_T("threshold"), MRT.threshold2->GetDouble());

  if(MRT.CT2.Camera)
    if(MRT.CT2.Camera->GetWidth()>0 && MRT.CT.Camera->GetHeight()>0)
      {
      pConfig->Write(_T("width"), MRT.CT2.Camera->GetWidth());
      pConfig->Write(_T("height"), MRT.CT2.Camera->GetHeight());
      pConfig->Write(_T("depth"), MRT.CT2.Camera->GetDepth());
      if(MRT.CT2.Camera->GetFrameRate()>0)
        pConfig->Write(_T("framerate"), MRT.CT2.Camera->GetFrameRate());
      }
   if(camCanvas2)
      pConfig->Write(_T("viewmode"), camCanvas2->GetPreviewStatus());
   pConfig->SetPath("../"); 


    // save a frame position
  int x, y, w, h;
  GetSize(&w, &h);
  GetPosition(&x, &y);
  pConfig->Write(_T("/MainFrame/x"), x);
  pConfig->Write(_T("/MainFrame/y"), y);
  pConfig->Write(_T("/MainFrame/w"), w);
  pConfig->Write(_T("/MainFrame/h"), h);
  
  IniFileEmpty = false;
  pConfig->Flush(true); ///< store a value into configuration file  
}


void BnPMainFrame::OnReloadConfig(wxCommandEvent & WXUNUSED(event))
{
  wxConfigBase *pConfig = wxConfigBase::Get();
  if (pConfig==NULL) return;

  if(IniFileEmpty)
    if(!pConfig->HasGroup("/HwSetting") || !pConfig->HasGroup("/MainFrame") 
		/* || !pConfig->HasGroup(DEFAULT_PID_GROUP)*/)
      {    
      wxMessageBox(_T("File camera.ini does not exist,\n"
                      "or there are missing items inside camera.ini."),
	 		APP_NAME, wxOK|wxICON_EXCLAMATION);
      return;
      }

  OnLoadConfig();
}


/** Load all configuretion stuff. */
void BnPMainFrame::OnLoadConfig()
{
long period;
int Width=DEFAULT_VIDEO_WIDTH;	  ///< Default camera video resolution - width
int Height=DEFAULT_VIDEO_HEIGHT;  ///< Default camera video resolution - height
int Depth=24;			  ///< Default camera depth
double FrameRate=30;
CamRtExec::CTR_MODES OldControlMode;

  wxConfigBase *pConfig = wxConfigBase::Get();
  if (pConfig==NULL) return;

  period =10;

  // load a sampling period
//  period = pConfig->Read(_T("/HwSetting/timer"), HwSetting::DEF_TIME);
  MRT.SetPeriodMs(period); 

  // load camera properties
  pConfig->SetPath("/HwSetting/");
  if(MRT.threshold)
      MRT.threshold->SetDouble(pConfig->Read(_T("threshold"), 
		CameraTool::DEFAULT_THR));
  pConfig->SetPath("../");

  pConfig->SetPath("/camera1/");
  if(MRT.CT.Camera)
    {  
    Width=pConfig->Read("width", DEFAULT_VIDEO_WIDTH);
    Height=pConfig->Read("height", DEFAULT_VIDEO_HEIGHT);
    Depth=pConfig->Read("depth", Depth);
    FrameRate=pConfig->Read("framerate", 0.0);	// 0 means default value
        
    if(Width!=MRT.CT.Camera->GetWidth() || Height!=MRT.CT.Camera->GetHeight() ||
       Depth!=MRT.CT.Camera->GetDepth() || 
       (abs(FrameRate-MRT.CT.Camera->GetFrameRate())>1e-3 && FrameRate>0))
      {
      MRT.CT.Camera->Stop();
      MRT.CT.Camera->SelectVideoMode(Width,Height,Depth,FrameRate);
      MRT.CT.Camera->Start();

	 /* Emit event about resolution change. */
      wxCommandEvent EvtDummy(wxEVT_CAMERARESCHANGED,GetId());
      OnResolutionChange(EvtDummy);
	     /* Traverse through all opened VideoProperty - "event" don't go to childs. */
      std::list<VideoSetting *>::iterator IterVideoProp;
      for(IterVideoProp=VideoSetting::Instances.begin(); IterVideoProp!=VideoSetting::Instances.end(); IterVideoProp++)
        (*IterVideoProp)->OnResolutionChanged(EvtDummy);
      }  
    }
  camCanvas->SetPreviewStatus(
      (VISION_STATES)pConfig->Read(_T("viewmode"),camCanvas->GetPreviewStatus()));
  pConfig->SetPath("../");


  pConfig->SetPath("/camera2/");
  if(MRT.CT2.Camera)
    {  
    Width=pConfig->Read("width", DEFAULT_VIDEO_WIDTH);
    Height=pConfig->Read("height", DEFAULT_VIDEO_HEIGHT);
    Depth=pConfig->Read("depth", 8);
    FrameRate=pConfig->Read("framerate", 0.0);	// 0 means default value
        
    if(Width!=MRT.CT2.Camera->GetWidth() || Height!=MRT.CT2.Camera->GetHeight() ||
       Depth!=MRT.CT2.Camera->GetDepth() || 
       (abs(FrameRate-MRT.CT2.Camera->GetFrameRate())>1e-3 && FrameRate>0))
      {
      MRT.CT2.Camera->Stop();
      MRT.CT2.Camera->SelectVideoMode(Width,Height,Depth,FrameRate);
      MRT.CT2.Camera->Start();

	 /* Emit event about resolution change. */
      wxCommandEvent EvtDummy(wxEVT_CAMERARESCHANGED,GetId());
      OnResolutionChange(EvtDummy);
	     /* Traverse through all opened VideoProperty - "event" don't go to childs. */
      std::list<VideoSetting *>::iterator IterVideoProp;
      for(IterVideoProp=VideoSetting::Instances.begin(); IterVideoProp!=VideoSetting::Instances.end(); IterVideoProp++)
        (*IterVideoProp)->OnResolutionChanged(EvtDummy);
      }    
    }
  camCanvas2->SetPreviewStatus(
      (VISION_STATES)pConfig->Read(_T("viewmode"),camCanvas2->GetPreviewStatus()));
  pConfig->SetPath("../");

  switch(camCanvas->GetPreviewStatus())
  {
    case RAW_PIC:         CamGray->Check();  break;
    case THRESHOLDED_PIC: CamBW->Check();    break;
    case RAW_COLOR:	  CamColor->Check(); break;
    default: return;  //no capture needed
  }
  switch(camCanvas2->GetPreviewStatus())
  {
    case RAW_PIC:         CamGray2->Check(); break;
    case THRESHOLDED_PIC: CamBW2->Check();   break;
    case RAW_COLOR:	  CamColor2->Check();   break;
    default: return;  //no capture needed
  }

//   if(OldControlMode>=CamRtExec::CTR_RUN_1ST) MRT.StartControl();
}


/** Shrink combo box, when camerachanges mode. */
void BnPMainFrame::OnResolutionChange(wxCommandEvent & event)
{
int ComboCount;

  event.Skip();		//This event should be spread widely, don't remove.

  if(MRT.CT.Camera)
    {
    if(camCanvas)
      szitemChart->SetRatio(MRT.CT.Camera->GetWidth(), MRT.CT.Camera->GetHeight());

    if(camCanvas)
      szitemChart->SetRatio(MRT.CT.Camera->GetWidth(), MRT.CT.Camera->GetHeight());

      /* Emit event about resolution change. */
    if(m_bmpBkg!=NULL) // not fully initialised form?
      {
      SetAutoLayout(true);
      Layout();
      }

     ComboCount = camCanvas->GetPreviewStatus();
  //  ComboCount = itemComboBox->GetCount();
    if(MRT.CT.Camera->GetDepth()<24) 
      {					// B&W mode
      if(ComboCount==4)
        {
        //itemComboBox->Delete(3);
//        if(itemComboBox->GetSelection()==wxNOT_FOUND)
          {
	    //itemComboBox->Select(RAW_PIC);
	    //OnComboBox(event);		// Software Select() do not generate event, call its handler directly!
					// Even direct call does not work!
	      camCanvas->SetPreviewStatus(RAW_PIC);	
          }
        }
      }
    else					// color mode
      {
//    if(ComboCount==3)
//	itemComboBox->Append(itemComboBoxStrings[3]);
      }
  }


  if(MRT.CT2.Camera)
    {
    if(camCanvas2)
      szitemChart2->SetRatio(MRT.CT2.Camera->GetWidth(), MRT.CT2.Camera->GetHeight());

    if(camCanvas2)
      szitemChart2->SetRatio(MRT.CT2.Camera->GetWidth(), MRT.CT2.Camera->GetHeight());

      /* Emit event about resolution change. */
    if(m_bmpBkg!=NULL) // not fully initialised form?
      {
      SetAutoLayout(true);
      Layout();
      }

     ComboCount = camCanvas2->GetPreviewStatus();
  //  ComboCount = itemComboBox->GetCount();
    if(MRT.CT2.Camera->GetDepth()<24) 
      {					// B&W mode
      if(ComboCount==4)
        {
        //itemComboBox->Delete(3);
//        if(itemComboBox->GetSelection()==wxNOT_FOUND)
          {
	    //itemComboBox->Select(RAW_PIC);
	    //OnComboBox(event);		// Software Select() do not generate event, call its handler directly!
					// Even direct call does not work!
	      camCanvas2->SetPreviewStatus(RAW_PIC);	
          }
        }
      }
    else					// color mode
      {
//    if(ComboCount==3)
//	itemComboBox->Append(itemComboBoxStrings[3]);
      }
  }

}


void BnPMainFrame::OnCameraChange(wxCommandEvent & event)
{
  if(camCanvas)
    camCanvas->OnCameraChange(event);
  if(camCanvas2)
    camCanvas2->OnCameraChange(event);      
}


/** Store image from buffer into a user selected file. */
void BnPMainFrame::OnLoad(wxCommandEvent & event)
{
unsigned y,x;
TImageFileHandler *LoadPicture_XXX;
wxString FileNames;

  for(LoadPicture_XXX=TImageFileHandler::first();LoadPicture_XXX!=NULL;LoadPicture_XXX=LoadPicture_XXX->next())
    {
    if(LoadPicture_XXX->LoadPicture==NULL) continue;

    const char *Ext = LoadPicture_XXX->extension();
    if(Ext==NULL) continue;
    if(*Ext==0) continue;
    if(*Ext == '.') Ext++;

    FileNames += Ext;
    FileNames += _T(" file (*.");
    FileNames += Ext;
    FileNames += _T(")|*.");
    FileNames += Ext;
    FileNames += _T("|");    
    }
 FileNames += _T("Any file (*)|*");

 wxFileDialog LoadDialog( this,
                       _T("Load image into buffer ..."),
                       wxEmptyString,
                       wxEmptyString,
                       FileNames,
                       0
                     );
  if (LoadDialog.ShowModal() == wxID_OK)
  {    
    Image I = LoadPicture(LoadDialog.GetPath());
    if(I.Raster==NULL)
      {
      wxMessageBox(_T("Image load failed or image is empty!"),
			APP_NAME, wxOK|wxICON_EXCLAMATION);
      return;
      }
    if(I.Raster->Size1D==0 || I.Raster->Size2D==0)
      {
      wxMessageBox(_T("Image is empty!"), APP_NAME, wxOK|wxICON_EXCLAMATION);
      return;
      }

    CamPaint->Check(true);    
    OnPaintStop(event);

    Raster2DAbstract *Ras = I.Raster;
    Raster2DAbstract *pImageBuf = camCanvas->GetImageBuf();

    if(Ras && pImageBuf)
      {
      pImageBuf->Allocate2D(Ras->Size1D,Ras->Size2D);
      
      for(y=0;y<Ras->Size2D;y++)
        {
	Ras->GetRowRaster(y)->Get24BitRGB(pImageBuf->GetRow(y));
        }

      if(Ras->GetPlanes()!=24 && pImageBuf->GetPlanes()==24)
        {
        for(y=0;y<pImageBuf->Size2D;y++)
          {
          BYTE *b = (BYTE *)pImageBuf->GetRowRaster(y)->GetRow();
          for(x=0;x<pImageBuf->Size1D;x++)
	    {
	    *b = b[1] = b[2];
	    b+=3;
	    }
	  }
        }
      }

      wxImage img = wxImage(pImageBuf->Size1D, pImageBuf->Size2D, (unsigned char *)pImageBuf->GetRow(0), true);
      camCanvas->SetImage(img);
      
      camCanvas->SetPreviewStatus(EXTERN_COLOR);
      
      camCanvas->Show();
      camCanvas->Refresh();
      if(szitemChart)  szitemChart->SetRatio(pImageBuf->Size1D, pImageBuf->Size2D);      
      SetAutoLayout(true);
      Layout();
  }
}

/** Store image from buffer into a user selected file. */
void BnPMainFrame::OnSaveAs(wxCommandEvent & event)
{
  TImageFileHandler *LoadPicture_XXX;
  wxString FileNames;
  for(LoadPicture_XXX=TImageFileHandler::first();LoadPicture_XXX!=NULL;LoadPicture_XXX=LoadPicture_XXX->next())
    {
    if(LoadPicture_XXX->SavePicture==NULL) continue;
    const char *Ext = LoadPicture_XXX->extension();
    if(Ext==NULL) continue;
    if(*Ext==0) continue;
    if(*Ext == '.') Ext++;

    if(LoadPicture_XXX->description())
      FileNames += LoadPicture_XXX->description();
    else
      FileNames += Ext;    
    FileNames += _T(" file (*.");
    FileNames += Ext;
    FileNames += _T(")|*.");
    FileNames += Ext;
    FileNames += _T("|");    
    }
  FileNames += _T("Any file (*)|*");

  wxFileDialog SaveDialog( this,
                       _T("Save image As ..."),
                       wxEmptyString,
                       wxEmptyString,
		       FileNames,
                       wxSAVE|wxOVERWRITE_PROMPT
                     );
  if (SaveDialog.ShowModal() == wxID_OK)
    {
    Image I;

    Raster2DAbstract *Ras = camCanvas->GetImageBuf();
    const unsigned char *pImageBuf = camCanvas->GetImageData();
    const unsigned char *raw_data = pImageBuf;
    if(raw_data==NULL) return;

    switch(camCanvas->GetPreviewStatus())
      {
      case THRESHOLDED_PIC:
		{
                I.Create(Ras->Size1D, Ras->Size2D, 1);
	        if(I.Raster==NULL) goto CANNOT_CREATE_RASTER;
                for(unsigned y=0;y<Ras->Size2D;y++)
                  for(unsigned x=0;x<Ras->Size1D;x++)
                    {
                    I.SetPixel(x,y, (*raw_data>128) ? 1 : 0);
	            raw_data+=3;
	            }
                break;
                }      
      case RAW_PIC:
		{
                I.Create(Ras->Size1D, Ras->Size2D, 8);
		if(I.Raster==NULL) goto CANNOT_CREATE_RASTER;
                for(unsigned y=0;y<Ras->Size2D;y++)
                  for(unsigned x=0;x<Ras->Size1D;x++)
                    {
                    I.SetPixel(x,y, *raw_data);
	            raw_data+=3;
	            }
                break;
                }      
      case RAW_COLOR:	
      default:	
		{
                I.Create(Ras->Size1D, Ras->Size2D, 8|ImageTrueColor);
		if(I.Raster==NULL) goto CANNOT_CREATE_RASTER;		  
                for(unsigned y=0;y<Ras->Size2D;y++)
		  {
		  I.Raster->GetRowRaster(y)->Set(*Ras->GetRowRaster(y));                  
                  }
                break;
                }
      }

    if(SavePicture(SaveDialog.GetPath(),I)<0)
       wxMessageBox(_T("Cannot save image into a file specified"),
			APP_NAME, wxOK|wxICON_EXCLAMATION);
    }
  return;

CANNOT_CREATE_RASTER:
  wxMessageBox(_T("Cannot create temporary image raster."), APP_NAME, wxOK|wxICON_EXCLAMATION);
  return;
}


/** Store image from buffer into a user selected file. */
void BnPMainFrame::OnSaveAs2(wxCommandEvent & event)
{
  TImageFileHandler *LoadPicture_XXX;
  wxString FileNames;
  for(LoadPicture_XXX=TImageFileHandler::first();LoadPicture_XXX!=NULL;LoadPicture_XXX=LoadPicture_XXX->next())
    {
    if(LoadPicture_XXX->SavePicture==NULL) continue;
    const char *Ext = LoadPicture_XXX->extension();
    if(Ext==NULL) continue;
    if(*Ext==0) continue;
    if(*Ext == '.') Ext++;
    
    if(LoadPicture_XXX->description())
      FileNames += LoadPicture_XXX->description();
    else
      FileNames += Ext;
    FileNames += _T(" file (*.");
    FileNames += Ext;
    FileNames += _T(")|*.");
    FileNames += Ext;
    FileNames += _T("|");
    }
  FileNames += _T("Any file (*)|*");
  wxFileDialog SaveDialog( this,
                       _T("Save image 2 As ..."),
                       wxEmptyString,
                       wxEmptyString,
		       FileNames,                       
                       wxSAVE|wxOVERWRITE_PROMPT
                     );
  if (SaveDialog.ShowModal() == wxID_OK)
    {
    Image I;

    const unsigned char *pImageBuf = camCanvas2->GetImageData();
    const unsigned char *raw_data = pImageBuf;
    Raster2DAbstract *Ras = camCanvas2->GetImageBuf();
    if(raw_data==NULL) return;

    switch(camCanvas2->GetPreviewStatus())
      {
      case THRESHOLDED_PIC:
		{
                I.Create(Ras->Size1D, Ras->Size2D, 1);
		if(I.Raster==NULL) goto CANNOT_CREATE_RASTER;
                for(unsigned y=0;y<Ras->Size2D;y++)
                  for(unsigned x=0;x<Ras->Size1D;x++)
                    {
                    I.SetPixel(x,y, (*raw_data>128) ? 1 : 0);
	            raw_data+=3;
	            }
                break;
                }      
      case RAW_PIC:
		{
                I.Create(Ras->Size1D, Ras->Size2D, 8);
		if(I.Raster==NULL) goto CANNOT_CREATE_RASTER;		
                for(unsigned y=0;y<Ras->Size2D;y++)
                  for(unsigned x=0;x<Ras->Size1D;x++)
                    {
                    I.SetPixel(x,y, *raw_data);
	            raw_data+=3;
	            }
                break;
                }      
      case RAW_COLOR:	
      default:	
		{
                I.Create(Ras->Size1D, Ras->Size2D, 8|ImageTrueColor);
		if(I.Raster==NULL) goto CANNOT_CREATE_RASTER;		
                for(unsigned y=0;y<Ras->Size2D;y++)
		  I.Raster->GetRowRaster(y)->Set(*Ras->GetRowRaster(y));                  
                break;
                }
      }

    if(SavePicture(SaveDialog.GetPath(),I)<0)
       wxMessageBox(_T("Cannot save image into a file specified"), APP_NAME, wxOK|wxICON_EXCLAMATION);
    }
  return;

CANNOT_CREATE_RASTER:
  wxMessageBox(_T("Cannot create temporary image raster."), APP_NAME, wxOK|wxICON_EXCLAMATION);
  return;
}



void BnPMainFrame::OnBlackWhite(wxCommandEvent & event)
{
  Switch(THRESHOLDED_PIC);
}

void BnPMainFrame::OnGray(wxCommandEvent & event)
{
  Switch(RAW_PIC);
}

void BnPMainFrame::OnColor(wxCommandEvent & event)
{
  Switch(RAW_COLOR);
}

/**  
 *  Manages the position view, video capture view  and thresholed view of the model.
 *  @param[in] event Unused event */
void BnPMainFrame::Switch(VISION_STATES choice)
{
  if(MRT.CT.Camera==NULL)
    {
    if(choice!=RAW_PIC)
      {
      camCanvas->SetPreviewStatus(RAW_PIC);
      //itemComboBox->Select(GRAPH);
      }
    return;
    }

  switch(choice)
    {
    case GRAPH: camCanvas->SetPreviewStatus(GRAPH);	   
	    break;
    case RAW_PIC:camCanvas->SetPreviewStatus(RAW_PIC);	    
	    break;
    case THRESHOLDED_PIC:
            camCanvas->SetPreviewStatus(THRESHOLDED_PIC);            
	    break;
    case RAW_COLOR:
	    camCanvas->SetPreviewStatus(RAW_COLOR);            
	    if(MRT.CT.Camera)
              if(MRT.CT.Camera->GetDepth()<24)
		wxMessageBox(_T("Camera must be switched to color mode.\n"
			"Please open dialog MainMenu/Settings/Camera 1/ChangeResolution and switch mode."),
			APP_NAME, wxOK|wxICON_EXCLAMATION);
	    break;
    }
  camCanvas->Refresh(); // ???
}


void BnPMainFrame::OnBlackWhite2(wxCommandEvent & event)
{
  Switch2(THRESHOLDED_PIC);
}

void BnPMainFrame::OnGray2(wxCommandEvent & event)
{
  Switch2(RAW_PIC);
}

void BnPMainFrame::OnColor2(wxCommandEvent & event)
{
  Switch2(RAW_COLOR);
}


/**  
 *  Manages the position view, video capture view  and thresholed view of the model.
 *  @param[in] event Unused event */
void BnPMainFrame::Switch2(VISION_STATES choice)
{
  if(MRT.CT2.Camera==NULL)
    {
    if(choice!=RAW_PIC)
      {
      camCanvas2->SetPreviewStatus(RAW_PIC);
      //itemComboBox->Select(GRAPH);
      }
    return;
    }

  switch(choice)
    {
    case GRAPH: camCanvas2->SetPreviewStatus(GRAPH);	   
	    break;
    case RAW_PIC:camCanvas2->SetPreviewStatus(RAW_PIC);	    
	    break;
    case THRESHOLDED_PIC:
            camCanvas2->SetPreviewStatus(THRESHOLDED_PIC);            
	    break;
    case RAW_COLOR:
	    camCanvas2->SetPreviewStatus(RAW_COLOR);            
	    if(MRT.CT2.Camera)
              if(MRT.CT2.Camera->GetDepth()<24)
		wxMessageBox(_T("Camera 2 must be switched to color mode.\n"
			"Please open dialog MainMenu/Settings/Camera 2/ChangeResolution and switch mode."),
			APP_NAME, wxOK|wxICON_EXCLAMATION);
	    break;
    }
  camCanvas2->Refresh(); // ???
}


/** Display a video setting dialog for camera1. */
void BnPMainFrame::OnVideo(wxCommandEvent& WXUNUSED(event))
{
  if(!camCanvas->IsShown())
    {
    camCanvas->Show();
    SetAutoLayout(true);
    Layout();
    }
  if(!VideoSetting::Instances.empty())	// Is a dialog already opened?
    {
    std::list<VideoSetting *>::iterator IterVideoProp;
    for(IterVideoProp=VideoSetting::Instances.begin(); IterVideoProp!=VideoSetting::Instances.end(); IterVideoProp++)
      {
      if((*IterVideoProp)->CT == &MRT.CT)
        {
        (*IterVideoProp)->SetFocus();
        return;
        }
      }    
    }
  VideoSetting *VideoSet = new VideoSetting(this,&MRT.CT);
  VideoSet->Show();
}

/** Display a video setting dialog for camera2. */
void BnPMainFrame::OnVideo2(wxCommandEvent& WXUNUSED(event))
{
  if(!camCanvas2->IsShown())
    {
    camCanvas2->Show();
    SetAutoLayout(true);
    Layout();
    }
  if(!VideoSetting::Instances.empty())	// Is a dialog already opened?
    {
    std::list<VideoSetting *>::iterator IterVideoProp;
    for(IterVideoProp=VideoSetting::Instances.begin(); IterVideoProp!=VideoSetting::Instances.end(); IterVideoProp++)
      {
      if((*IterVideoProp)->CT == &MRT.CT2)
        {
        (*IterVideoProp)->SetFocus();
        return;
        }
      }    
    }
  VideoSetting *VideoSet = new VideoSetting(this,&MRT.CT2);
  VideoSet->Show();

  if(camCanvas)
    if(camCanvas->IsShown())
      {		// camCanvas must be refreshed otherwise it is not repainted correctly
      camCanvas->Refresh(false);
      }
}


/** Opens plot frame for the graphs of signals
 *  @param[in] event Unused event  */
void BnPMainFrame::OnHistogram(wxCommandEvent& WXUNUSED(event))
{ 
PlotHistogram *pgf = NULL;

  if(!PlotHistogram::Instances.empty())	// Is a dialog already opened?
    {
    pgf = PlotHistogram::Instances.front();    
    }

  if(pgf==NULL)
    {
    pgf = new PlotHistogram(this);
/*
    pgf->ImgX=MRT.VideoX;	//assign real time points to be trended
    pgf->ImgY=MRT.VideoY;
*/
    pgf->TargetX=&MRT.ReqX;
    pgf->TargetY=&MRT.ReqY;
    pgf->u_X=&MRT.ControllerX;
    pgf->u_Y=&MRT.ControllerY;
    }
  pgf->Image = camCanvas->GetImageBuf();
  pgf->Show();
  pgf->SetFocus();
}


/** Stops the timer */
void BnPMainFrame::StopCapturing(void)
{
  if(mPreviewTimer.IsRunning())
           mPreviewTimer.Stop();
};


/** Starts the timer. */
void BnPMainFrame::StartCapturing(void)
{
 if(!mPreviewTimer.IsRunning())
     mPreviewTimer.Start();
};


/**
 *  Stops the controller, but timer still paints image
 *  @param[in] event Unused event   */
void BnPMainFrame::OnPaintStop(wxCommandEvent& WXUNUSED(event))
{
  if(CamPaint->IsChecked())
    SetStatusText(_T("Painting mode off."));
  else
    {
    SetStatusText(_T("Painting mode on."));
    if(camCanvas->GetPreviewStatus()==EXTERN_COLOR)
      {
      camCanvas->SetPreviewStatus(RAW_COLOR);
      if(szitemChart) szitemChart->SetRatio(MRT.CT.GetWidth(), MRT.CT.GetHeight());
      SetAutoLayout(true);
      Layout();
      Refresh();
      }
    }
}


void BnPMainFrame::OnPaintFilter(wxCommandEvent& WXUNUSED(event))
{
  if(camCanvas==NULL) return;

  if(CamFilter->IsChecked())
    camCanvas->filter = 1;
  else
    camCanvas->filter = 0;
}


void BnPMainFrame::OnRectangle(wxCommandEvent& WXUNUSED(event))
{
  if(camCanvas==NULL) return;

  if(CamRectangle->IsChecked())
    camCanvas->rectangle = 1;
  else
    camCanvas->rectangle = 0;

  camCanvas->FlushFilter();
  camCanvas->CaptureImage(); 
  szitemChart->SetRatio(MRT.CT.GetWidth(), MRT.CT.GetHeight());
  szitemChart2->SetRatio(MRT.CT2.GetWidth(), MRT.CT2.GetHeight());
  SetAutoLayout(true);
  Layout();
  Refresh();
}

void BnPMainFrame::OnPaintFilter2(wxCommandEvent& WXUNUSED(event))
{
  if(camCanvas2==NULL) return;

  if(CamFilter2->IsChecked())
    camCanvas2->filter = 1;
  else
    camCanvas2->filter = 0;
}


void BnPMainFrame::OnRectangle2(wxCommandEvent& WXUNUSED(event))
{
  if(camCanvas2==NULL) return;

  if(CamRectangle2->IsChecked())
    camCanvas2->rectangle = 1;
  else
    camCanvas2->rectangle = 0;

  camCanvas2->FlushFilter();
  camCanvas2->CaptureImage();
  if(szitemChart)  szitemChart->SetRatio(MRT.CT.GetWidth(), MRT.CT.GetHeight());
  if(szitemChart2) szitemChart2->SetRatio(MRT.CT2.GetWidth(), MRT.CT2.GetHeight());
  SetAutoLayout(true);
  Layout();
  Refresh();
}

/**
 *  Stops the controller, but timer still paints image
 *  @param[in] event Unused event   */
void BnPMainFrame::OnPaintStop2(wxCommandEvent& WXUNUSED(event))
{
  if(CamPaint2->IsChecked())
    SetStatusText(_T("Painting mode 2 off."));
  else
    SetStatusText(_T("Painting mode 2 on."));
}



/** Initializes all the values to the default one.
 *  @param[in] event Unused event  */
void BnPMainFrame::OnSetDefault(wxCommandEvent& WXUNUSED(event))
{
	//reset timer to default
//  MRT.SetPeriodMs(HwSetting::DEF_TIME);
        //reset video to default
  if(MRT.threshold!=NULL)
    MRT.threshold->SetDouble(CameraTool::DEFAULT_THR);

  camCanvas->Reset();
}


/**
 *  When the size is changed, it is called and then it refreshs the screen and sets the layouts   
 *
 *  @param[in] event Unused event
 *
 *  \warning If the drawing depends on the size of the window, 
 *  \warning clearing the DC explicitly may be needed and repainting the whole window. 
 *  \warning In which case, calling <CODE>wxWindow::Refresh()</CODE> may be needed to invalidate the entire window.  */
void BnPMainFrame::OnSize(wxSizeEvent& WXUNUSED(event))
{   // if your drawing depends on the size of the window, 
    // you may need to clear the DC explicitly and repaint the whole window. 
    // In which case, you may need to call wxWindow::Refresh() to invalidate the entire window.
  SetAutoLayout(true);
  Layout();
  Refresh();
}


/** When a size is changed, it is called and then it refreshs the screen and sets the layouts 
 *  @param[in] event Unused event
 *  \warning If the drawing depends on the size of the window, 
 *  \warning clearing the DC explicitly may be needed and repainting the whole window. 
 *  \warning In which case, calling <CODE>wxWindow::Refresh()</CODE> may be needed to invalidate the entire window. */
void BnPMainFrame::OnEraseBackground(wxEraseEvent& WXUNUSED(event))
{
 // do nothing
}

