/********************************************************************/
/** @file CamView/PlotHistogram.cpp
*/

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
 #include <wx/wx.h>
#endif

#include "../library/hardware/Timer.h"
#include "PlotHistogram.h"
#include "RT_EXEC.h"

#define REPAINT_TIMER_MS 691	///< Period of repainting chart. A chart is full repainted only when it is marked as dirty.


std::list<PlotHistogram *> PlotHistogram::Instances; ///< List of opened chart objects.


DEFINE_EVENT_TYPE(wxEVT_FORMCLOSED)

/**
 * Constructor
 * @param[in] parent the parent wxWindow pointer.  */
PlotHistogram::PlotHistogram(wxFrame *parent):
    wxFrame(parent, -1, "Histogram", wxDefaultPosition, wxSize(675, 550), wxDEFAULT_FRAME_STYLE),    
    mTimer(this, TIMER_ID),
    TmrRepaint(this, ID_REPAINT),
    TargetX(NULL), TargetY(NULL), ImgY(NULL),
    u_Y(NULL),
    MnuHold(NULL),
    FilterData(false)
{    
    Instances.push_back(this);
    RealTime.SetDouble(0);

    // Make a menubar
    wxMenuBar *menuBar = new wxMenuBar;    

          // now append the freshly created menu to the menu bar...
    // create Ranges
    menuClose = new wxMenu;
    menuClose->Append(ID_MENU_CLOSE, _T("&Close Chart"));
    menuBar->Append(menuClose, _T("&Close"));  // append menu item to menubar

    wxMenu *menuRange = new wxMenu;
    menuRange->Append(ID_MENU_RANGE, _T("&Range of Time Axis"));            
    menuRange->AppendSeparator();
/*
    wxMenuItem *MnuSmoothAct = 
       new wxMenuItem(menuRange,ID_MENU_SMOOTH_ACTION, _T("&Smooth Action Curve"), _T("Filter values inside action curve."),wxITEM_CHECK);
    menuRange->Append(MnuSmoothAct);    
    if(FilterData) MnuSmoothAct->Check();
*/
/*    wxMenuItem *HideAct = 
       new wxMenuItem(menuRange,ID_MENU_HIDE_ACTION, _T("&Hide Action Curve"), _T("Action curve is not painted."),wxITEM_CHECK);
    menuRange->Append(HideAct);    
    HideAct->Check();
*/
    //if(!FilterData) MnuSmoothAct->Check();

    menuBar->Append(menuRange, _T("&Range"));  // append menu item to menubar
    
    wxMenu *menuMode = new wxMenu;
    menuBar->Append(menuMode, _T("&Paint Mode"));  // append menu item to menubar
        
    	//a default item must be placed first - no other check is made
    menuMode->AppendRadioItem(ID_MENU_MODE_FLINE, _T("&Fixed Fast Repaint"), _T("Current time is placed on a cursor."));
    menuMode->AppendRadioItem(ID_MENU_MODE_FLOW, _T("&Moving"),_T("Current sample is placed on right axis."));
    //menuMode->AppendRadioItem(ID_MENU_MODE_LINE, _T("&Fixed"));    

    wxMenu *menuClear = new wxMenu;
    menuClear->Append(ID_MENU_CLEAR, _T("&Clear Chart"), _T("Erase all chart's data.") );
    menuBar->Append(menuClear, _T("&Clear"));  // append menu item to menubar   

    wxMenu *menuHold = new wxMenu;
    MnuHold = new wxMenuItem(menuHold,ID_MENU_HOLD,_T("&Hold Chart"),_T("Stop painting chart."),wxITEM_CHECK);
    menuHold->Append(MnuHold);    
    menuBar->Append(menuHold, _T("&Hold"));  // append menu item to menubar   


		// Associate the menu bar with the frame
    SetMenuBar(menuBar);
    CreateStatusBar();
    SetStatusText(_T("Legend: green-position red-setpoint cyan-action"));
    
    this->SetBackgroundColour(wxColour(0, 0, 0));    
   
    plotA = new Plotter(this,wxT("")); //, wxT("Histogram")/*, graphLineNumberA */ ); /* Number of Graph line inside the panel */    

    wxColour lineColor = wxColour("RED"); 
    wxPen pen = wxPen(lineColor, 2, wxSOLID);
    plotA->SetSignalLine(2, pen);    

    lineColor = wxColour("GREEN");
    pen = wxPen(lineColor, 2, wxSOLID);
    plotA->SetSignalLine(1, pen);    

    lineColor = wxColour("CYAN");
    pen = wxPen(lineColor, 2, wxSOLID);
    plotA->SetSignalLine(0, pen);

    double Yrange[2]={0,1};
    plotA->SetYrange(Yrange);
    

    double Xrange[2]={0,255};
    plotA->SetXrange(Xrange);

    plotA->SetXTicks(10);
    plotA->SetXLblTicks(5);
    plotA->SetYTicks(10);
    plotA->SetYLblTicks(5);

    //plotA->SerVisible(0,!HideAct->IsChecked());    
 
    //wxColour backgColor = wxColour(100, 0, 123);
    //plotB->SetBackground(wxColour("BLACK"));

	/* Create layout of chart */
    Szr_Chart = new wxBoxSizer(wxVERTICAL);    

    Szr_Chart->Add(plotA, 1, wxEXPAND|wxALL, 0);    

    const wxSize& minSize = wxSize(350, 350); 
    SetSizeHints(minSize);
    SetSizer(Szr_Chart);

    SetIcon(parent->GetIcon());  //prefer same icons for all opened frames

    SetAutoLayout(true);
    Layout();
    //Fit();
}


/** destructor emits wxEVT_FORMCLOSED event, that could be catched from parent forms. */
PlotHistogram::~PlotHistogram()
{
  Instances.remove(this);

  if(TmrRepaint.IsRunning())
    TmrRepaint.Stop();
  if(mTimer.IsRunning())
    mTimer.Stop();

  plotA->Clear();

     /* Emit event from destructor. */
  wxCommandEvent event(wxEVT_FORMCLOSED,GetId());
  event.SetEventObject(this);  
//  event.SetText( wxT("Destroy") );  // Give it some contents
  GetEventHandler()->ProcessEvent(event);

  if(plotA)
    {delete plotA;plotA=NULL;}
  //delete plotB;
}


BEGIN_EVENT_TABLE(PlotHistogram, wxFrame)
    EVT_TIMER(TIMER_ID,         PlotHistogram::OnGrabTimer)
    EVT_TIMER(ID_REPAINT,       PlotHistogram::OnRepaintTimer)
    EVT_ERASE_BACKGROUND(       PlotHistogram::OnEraseBackground)

    EVT_BUTTON(ID_BUTTON_CLEAR, PlotHistogram::OnBtnClear)
    EVT_BUTTON(ID_BUTTON_CLOSE, PlotHistogram::OnBtnClose)    
    EVT_SIZE(                   PlotHistogram::OnSize)
    EVT_MENU(ID_MENU_RANGE,     PlotHistogram::OnRangeEntry)
    EVT_MENU(ID_MENU_SMOOTH_ACTION,PlotHistogram::OnSmoothAction)
    EVT_MENU(ID_MENU_HIDE_ACTION,PlotHistogram::OnHideAction)
    EVT_MENU(ID_MENU_CLOSE,     PlotHistogram::OnBtnClose)
//    EVT_MOTION(                 PlotHistogram::OnMouseMove)        
    EVT_ACTIVATE(		PlotHistogram::OnActivate)
    EVT_CLOSE(			PlotHistogram::OnClose)
    EVT_MENU(ID_MENU_MODE_FLOW, PlotHistogram::OnMnuFlow)
    EVT_MENU(ID_MENU_MODE_LINE, PlotHistogram::OnMnuLine)
    EVT_MENU(ID_MENU_MODE_FLINE,PlotHistogram::OnMnuFLine)
    EVT_MENU(ID_MENU_CLEAR,     PlotHistogram::OnBtnClear)
    EVT_MENU(ID_MENU_HOLD,      PlotHistogram::OnMnuHold)    

    EVT_MENU_OPEN(              PlotHistogram::OnMenuOpen)    
END_EVENT_TABLE()


void PlotHistogram::OnActivate(wxActivateEvent & WXUNUSED(event))
{
  if(!MnuHold->IsChecked())
    {    
    if(!TmrRepaint.IsRunning())
        TmrRepaint.Start(REPAINT_TIMER_MS);    
    }
  else
    {
    if(TmrRepaint.IsRunning())
        TmrRepaint.Stop();    
    }
}


void PlotHistogram::OnClose(wxCloseEvent & WXUNUSED(event))
{
  TmrRepaint.Stop(); 
  mTimer.Stop();
  Show(false);  
  delete this;
}



/** Clears the plot window
 *  @param[in] event Unused event   */
void PlotHistogram::OnBtnClear(wxCommandEvent& WXUNUSED(event))
{
  plotA->Clear();  
  plotA->Refresh();  
}



void PlotHistogram::OnMnuLine(wxCommandEvent & WXUNUSED(event))
{
  plotA->SetPaintMode(0);
  plotA->Refresh(false);
}


void PlotHistogram::OnMnuFLine(wxCommandEvent & WXUNUSED(event))
{
  plotA->SetPaintMode(2);
  plotA->Refresh(false);  
}


void PlotHistogram::OnMnuFlow(wxCommandEvent & WXUNUSED(event))
{
  plotA->SetPaintMode(1);
  plotA->Refresh(false);
}



/** Close button - only hides a PlotHistogram window, it lives further ?!?
 *  @param[in] event Unused event   */
void PlotHistogram::OnBtnClose(wxCommandEvent& event)
{
  event.StopPropagation();

  if(TmrRepaint.IsRunning())
    TmrRepaint.Stop();

  wxCommandEvent PlotCloseEvent(wxEVT_FORMCLOSED,GetId());
  PlotCloseEvent.SetEventObject( this );  
//event.SetText( wxT("Destroy") );  // Give it some contents
  GetEventHandler()->ProcessEvent(PlotCloseEvent);

  Show(false);  
}


void PlotHistogram::OnSmoothAction(wxCommandEvent& event)
{   
 FilterData = event.IsChecked();
}


void PlotHistogram::OnHideAction(wxCommandEvent& event)
{   
 plotA->SerVisible(0,!event.IsChecked());
 plotA->Refresh(false);
}


/**
 *  Takes a value via wxTextEntryDialog() for the graph's time axis
 *  @param[in] event from wxWindows
 */
void PlotHistogram::OnRangeEntry(wxCommandEvent& event)
{   
    if (event.IsCommandEvent())
    {
        double value[2];
        plotA->GetXrange(value);
        wxTextEntryDialog dialog(this,
                                _T("Range of time axis[s]:\n"),
                                _T("Set Time Range"),
                                wxString::Format(wxT("%.2f"), value[1]),
                                wxOK | wxCANCEL);
        dialog.SetIcon(GetIcon());

        if (dialog.ShowModal() == wxID_OK)
        {   
            wxString valText = dialog.GetValue();
            //wxMessageBox(valText, _T("Got string"), wxOK | wxICON_INFORMATION, this); //debugging
            double val;
            if( !valText.ToDouble(&val) || val <= 0)
            {    
                wxMessageBox(_T("Invalid entry was typed\n") , _T("Error"), wxOK | wxICON_ERROR, this);
                OnRangeEntry(event);
                return;
            }
            value[1] = val;
            plotA->SetXrange(value);
            if(val==9)
              {
	      plotA->SetXTicks(9);
              plotA->SetXLblTicks(3);
	      }
            else if(val==8 || val==4)
              {
	      plotA->SetXTicks(8);
              plotA->SetXLblTicks(4);
	      }
	    else if(val==6 || val==12)
              {
	      plotA->SetXTicks(12);
              plotA->SetXLblTicks(6);
	      }
            else
	      {
	      plotA->SetXTicks(10);
              plotA->SetXLblTicks(5);
	      }
        }
    }
    else
        event.Skip();
}


/**
 * When the size is changed, it is called and then it refreshs the screen and sets the layouts   
 * @param[in] event Unused event
 *  \warning If your drawing depends on the size of the window, 
 *  \warning you may need to clear the DC explicitly and repaint the whole window. 
 *  \warning In which case, you may need to call wxWindow::Refresh() to invalidate the entire window. */
void PlotHistogram::OnSize(wxSizeEvent& WXUNUSED(event))
{
  Refresh();
  SetAutoLayout(true);
  Layout();
}


/** Do whatever you want to do every second here
 * Uses for testing graph
 * @param[in] event Unused event 
  \todo - this should be rewritten to be event driven */
void PlotHistogram::OnGrabTimer(wxTimerEvent &WXUNUSED(event))
{
double Values[4];
unsigned ElToPaint;
unsigned Dilution=1;
static double OldVal;
    // do whatever you want to do every period of timer here    

/* !!!!!!!!!!
  ElToPaint = MRT.CB_Time.Elements();
  
  if(ElToPaint>220) Dilution=2; 
  if(ElToPaint>420) Dilution=3; 
  if(ElToPaint>620) Dilution=4;
  if(ElToPaint>820) Dilution=5;
  if(ElToPaint>920) Dilution=6;
  if(ElToPaint==0) 
    {
    ElToPaint++;
    MRT.PumpRawCB();    
    }  

  for(unsigned i=0;i<ElToPaint;i++)
    {
    Values[0] = MRT.CB_Time.Pop();
    Values[1] = MRT.CB_OutputMag.Pop();
    Values[2] = MRT.CB_BallPos.Pop();		   // Circle X-axis
    Values[3] = MRT.CB_Req.Pop();		   // Setpoint X-axis

    if(FilterData && ElToPaint>1)      
      Values[1] = 0.9*OldVal + 0.1*Values[1];	  // action          

    if(i%Dilution == 0 || i>=ElToPaint-1)
      plotA->AddPoint(4,Values,i>=ElToPaint-1);

    OldVal = Values[1];
    }  
*/

 //wxString str;
 //str.Printf( wxT("Elements: %d %d"), ElToPaint, MRT.CB_Time.Elements() );
 //SetStatusText( str );
}


/** Repaint both plots */
void PlotHistogram::OnRepaintTimer(wxTimerEvent & WXUNUSED(event))
{
double nRGB[256][4];
double Yrange[2]={1,0};
int i,j;

  //plotA->PaintMe();
  if(IsIconized()) return;	//do not waste time when iconised

  if(Image->GetPlanes()!=24) return;	//Other image types are not supported yet.

  plotA->Clear();

  for(i=0;i<255;i++)
    {
    nRGB[i][0] = i;
    nRGB[i][1] = nRGB[i][2] = nRGB[i][3] = 0;
    }

  for(i=0;i<Image->Size2D;i++)
    {
    unsigned char *ImgData = (unsigned char *)Image->GetRow(i);    

    for(j=0;j<Image->Size1D;j++)
      {      
      nRGB[*ImgData++][1] += 1;
      nRGB[*ImgData++][2] += 1;
      nRGB[*ImgData++][3] += 1;
      }
    }

  for(i=0;i<255;i++)
    {
    plotA->AddPoint(4, nRGB[i],false);

    if(Yrange[0]<nRGB[i][1]) Yrange[0]=nRGB[i][1];
    if(Yrange[0]<nRGB[i][2]) Yrange[0]=nRGB[i][2];
    if(Yrange[0]<nRGB[i][3]) Yrange[0]=nRGB[i][3];
    }

  //plotA->TxAux1.X.End = 255;
  //plotA->TxAux1.X.Start = 0;   
  
  plotA->SetYrange(Yrange);

  if(plotA->dirty) plotA->Refresh(false);    
//  if(plotB->dirty) plotB->Refresh(false);  
}


void PlotHistogram::OnMnuHold(wxCommandEvent & event)
{
  if(!event.IsChecked())
    {    
    if(!TmrRepaint.IsRunning())
        TmrRepaint.Start(REPAINT_TIMER_MS);    
    }
  else
    {
    if(TmrRepaint.IsRunning())
        TmrRepaint.Stop();    
    }
}


/**
 * Shows the mouse position
 * @param[in] event Unused event
 * \warning remove OnMouseMove() before submit  */
void PlotHistogram::OnMouseMove(wxMouseEvent& event)
{   
  wxClientDC dc(this);
  wxPoint pos = event.GetPosition();
  long x = dc.DeviceToLogicalX( pos.x );
  long y = dc.DeviceToLogicalY( pos.y );
  wxString str;
  str.Printf( wxT("Current mouse position: %d,%d"), (int)x, (int)y );
  SetStatusText( str );    
}


void PlotHistogram::OnMenuOpen(wxMenuEvent& event)
{
wxMenu *MNU = event.GetMenu();

  //wxString str;
  //str.Printf( wxT("PlotHistogram::MenuSel %d"), event.GetMenuId() );
  //SetStatusText( str );
  if(MNU==menuClose)
    {
    wxCommandEvent event2(wxEVT_FORMCLOSED,GetId());
    event2.SetEventObject(this);  
    OnBtnClose(event2);
    }
    
}


/** When a size is changed, it is called and then it refreshs the screen and sets the layouts 
 *  @param[in] event Unused event
 *  \warning If the drawing depends on the size of the window, 
 *  \warning clearing the DC explicitly may be needed and repainting the whole window. 
 *  \warning In which case, calling <CODE>wxWindow::Refresh()</CODE> may be needed to invalidate the entire window. */
void PlotHistogram::OnEraseBackground(wxEraseEvent & WXUNUSED(event))
{  
 // do not paint anything

  if(MnuHold)
    if(MnuHold->IsChecked()) return;

/*
  if(MRT.CB_Time.Elements()>100)    
    {
    wxTimerEvent evt;
    OnGrabTimer(evt);
    }
*/
}



