/*****************************************************************/
/**@file CameraTools.h: 
 * Desctiption: Higher level abstraction prototypes above raw    *
 *              camera driver.*					 *
 * Dependency:						  	 *
 *****************************************************************/
#ifndef _CAMERA_TOOLS_H_INCLUDED_
#define _CAMERA_TOOLS_H_INCLUDED_

#include <vector>

#include "points.h"


class CamImage
{
public:
  int Width;
  int Height;
  std::vector<unsigned char> Buffer; ///< Internal buffer for ball detector  

  inline void CheckX(int & XVal)     ///< Ensure proper range that fits horizontally
    {if(XVal<0) XVal=0; if(XVal>Width) XVal=Width-1;}
  inline void CheckY(int & YVal)     ///< Ensure proper range that fits vertically
    {if(YVal<0) YVal=0; if(YVal>Height) YVal=Height-1;}

  inline bool iswhite(int x, int y)
	{return(Buffer[Width*y + x]!=0);} 
};


/** Wrapper around camera grabber - suport for ball searching */
class CameraTool: public AdDriver
{
private:  
  DoublePoint VideoX;		///< Detected X video coordinate
  DoublePoint VideoY;		///< Detected Y video coordinate

  unsigned char threshold;	///< Threshold for image detection

  ByteWrapPoint WrThreshold;    ///< External access to threshold through point interface
  CamImage IntImage;		///< Internal buffer for ball detector  

  int iWidth, iHeight;

public:
  CameraTool(void): Camera(NULL), threshold(DEFAULT_THR),
		   WrThreshold(&threshold)	{}	///< default constructor
  
  bool ReadImageAs(std::vector<unsigned char> & ImageData, int mode);
  bool ReadImageAs(Raster2DAbstract *ImageBuf, int mode);

  CameraDevice *Camera;
  int GetWidth(void) {return iWidth;}		///< transformed width
  int GetHeight(void) {return iHeight;}	        ///< transformed height
    
// Interface AdDriver
  Point *GetInput(int Index);	                ///< all points are accessible through this interface

  static const int DEFAULT_THR;			///< Default threshold value

private:  
  	
public:    
  virtual int Execute(void);  

  /**Set a threshold value */
  void SetThreshold(unsigned char New_thr) {threshold=New_thr;}  
  unsigned char GetThreshold(void)  ///< Read a threshold value 
	{return(threshold);};


};

#endif // _CAMERA_TOOLS_H_INCLUDED_