/**@file wxCamera.cpp: 
   wx aware part of camera that allows user to change resolution. */

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
 #include <wx/wx.h>
#endif

#include "camera.h"
#include "../../wxTools.h"


/** Scan camera capabilities and select best video mode. */
void CameraInput::ChangeResolution(void)
{
TAudVidConfCaps *PAudVidConfCaps;
VIDEOINFOHEADER *pVIH;
wxArrayString ResolutionModes;
wxString Str;
HRESULT hr;
int CurrentMode;

  if(camera==NULL) return;
  if(!BuildGraph()) return;
 
  int count=sizeof(AUDIO_STREAM_CONFIG_CAPS);
  int size=sizeof(VIDEO_STREAM_CONFIG_CAPS);
  hr=camera->GetNumberOfCapabilities(&count, &size);
  if(FAILED(hr)) return;

  AM_MEDIA_TYPE* mt;
  int planes;  

  PAudVidConfCaps = (TAudVidConfCaps *)CoTaskMemAlloc(sizeof(TAudVidConfCaps));
  if(PAudVidConfCaps==NULL) return;

  CurrentMode=0; 
  mt=NULL;
  for (int i=0; i<count; i++)
    {      
      camera->GetStreamCaps(i, &mt, (BYTE*)PAudVidConfCaps);

      planes=(8*mt->lSampleSize)/(PAudVidConfCaps->v.InputSize.cx*PAudVidConfCaps->v.InputSize.cy);      
      if(planes==12) planes=8;

      if(Width==PAudVidConfCaps->v.InputSize.cx &&
         Height==PAudVidConfCaps->v.InputSize.cy &&
         planes==Planes) CurrentMode=i;

      Str.Printf(wxT("%dx%d: %d"),PAudVidConfCaps->v.InputSize.cx,PAudVidConfCaps->v.InputSize.cy,planes);
      if(planes==8) Str+=" Gray";
      if(planes==24) Str+=" Color";
      ResolutionModes.Add(Str);

      if(mt) {CoTaskMemFree(mt);mt=NULL;}
    }

   if(ResolutionModes.GetCount()>1)
    {
    int result = wxGetSingleChoiceIndex(_("Select video mode:"),
	   _("Available Video Modes"), CurrentMode, ResolutionModes, NULL //wxFindSuitableParent()
	   );
    if(result>=0)
      {   	//setup an user chosen format
      hr=camera->GetStreamCaps(result, &mt, (BYTE*)PAudVidConfCaps);
      if(SUCCEEDED(hr))
        {        
        pVIH=(VIDEOINFOHEADER*)mt->pbFormat;
        if(FrameDuration!=0) pVIH->AvgTimePerFrame=FrameDuration; //use a same frame rate

        hr=camera->SetFormat(mt); 
        if(mt->subtype==MEDIASUBTYPE_dvsl || mt->subtype==MEDIASUBTYPE_dvsd ||
           mt->subtype==MEDIASUBTYPE_dvhd 
           //mt->subtype==MEDIASUBTYPE_dv25 || mt->subtype==MEDIASUBTYPE_dv50 ||
           //mt->subtype==MEDIASUBTYPE_dvh1  //proffessional types, CLSID not found.
          )  
             {
	     UseFilter();
             mt->subtype = MEDIASUBTYPE_RGB24;
	     grabber->SetMediaType(mt);
             }

        if(SUCCEEDED(hr))
          {
          Width=PAudVidConfCaps->v.InputSize.cx;
          Height=PAudVidConfCaps->v.InputSize.cy;
          Planes=(8*mt->lSampleSize)/(PAudVidConfCaps->v.InputSize.cx*PAudVidConfCaps->v.InputSize.cy);
          if(Planes<8)
            {
            Planes = pVIH->bmiHeader.biPlanes*pVIH->bmiHeader.biBitCount;
            }
          if(Planes==12) Planes=8;	//12 bits means 8 + same garbage
                  
          FrameDuration = pVIH->AvgTimePerFrame;
          MinFrameDuration = PAudVidConfCaps->v.MinFrameInterval;
          MaxFrameDuration = PAudVidConfCaps->v.MaxFrameInterval;

          if(pVIH->bmiHeader.biCompression!=0 && filter_base==NULL && Planes<8)             
            {            
            wxMessageBox(
               _T("Compressed camera stream is not supported, please select another mode!"),
	       _T("Camera Viewer demo"), wxOK|wxICON_EXCLAMATION);
	    }
          }
        }
      if(FAILED(hr))
        {
        wxMessageBox(
               _T("Changing camera resolution failed, try it again!"),
	       _T("Camera Viewer demo"), wxOK|wxICON_EXCLAMATION);
        }
      }
    }

  if(PAudVidConfCaps) {CoTaskMemFree(PAudVidConfCaps);PAudVidConfCaps=NULL;}
  if(mt) {CoTaskMemFree(mt);mt=NULL;}
}